package gs.log.v0

import java.time.Instant
import java.time.LocalDate

/** Type class for data that can be logged by the `gs-log` library.
  */
trait Loggable[A]:
  /**
   * Render the given data as some primitive supported by `gs-log`. Note that
   * this type class does **not** support structured types.
   *
   * @param data The data to render.
   * @return The [[LogData.Value]] housing the output primitive.
   */
  def renderForLogs(data: A): LogData.Value

object Loggable:

  def apply[A](
    using
    L: Loggable[A]
  ): Loggable[A] = L

  given Loggable[String]  = LogData.Str(_)
  given Loggable[Boolean] = LogData.Bool(_)
  given Loggable[Int]     = LogData.Int32(_)
  given Loggable[Long]    = LogData.Int64(_)
  given Loggable[Float]   = LogData.Float32(_)
  given Loggable[Double]  = LogData.Float64(_)
  given Loggable[LocalDate]  = LogData.Date(_)
  given Loggable[Instant] = LogData.Instant(_)

end Loggable
